package config;

# Central P.A.T.H Configuration Module
#
# For more information please read the POD documentation
#
# Programmed by Bastian Ballmann [ bytebeater@crazydj.de ]
# http://www.crazydj.de
#
# Last Update: 22.12.2002
#
# This code is licensed under the GPL

###[ Loading modules ]###

use strict;
use Carp qw(croak);


###[ Klassenvariablen ]###

# Config Object
my (%obj,$obj,%opts);
my ($target,$source,$port);


# Create a new config object
sub new
{
    my $class = shift;
    $obj{'target'} = [];
    $obj{'source'} = [];
    $obj{'connection'} = [];
    $obj{'port'} = [];
    $obj{'multitarget'} = 0;
    $obj{'multisource'} = 0;
    $obj{'multiport'} = 0;
    $obj{'device'} = "eth0";
    $obj = bless \%obj, $class;
    return $obj;
}


# Check the configuration
sub check
{
    my $obj = shift;
    my %args = @_;
    my $code = 0;

# Device config
if($args{'i'})
{
    $obj->{'device'} = $args{'i'};
}



# Target Config
if($args{'h'})
{
    $target = $args{'h'};

# Target list
    if($target =~ /\,/g)
    {
	@{$obj->{'targetlist'}} = split(/\,/,$target);
	$obj->{'multitarget'} = 1;
    }

# Target range
    elsif($target =~ /\-/)
    {
	my ($start,$stop) = split(/\-/,$target);

	my $result = check_ip($start);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $start\n\n";
	    $code = 1;
	}

	$result = check_ip($stop);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $stop\n\n";
	    $code = 1;
	}

	@{$obj->{targetlist}} = get_ips($start,$stop);
	$obj->{'multitarget'} = 1;
    }

# Einzelnes Target
    else
    {
	my $result = check_ip($target);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $target\n\n";
	    $code = 1;
	}

	$obj->{'targetlist'}->[0] = $target;
    }
}


# Source Config
if($args{'s'})
{
    $source = $args{'s'};

# Source list
    if($source =~ /\,/g)
    {
	@{$obj->{'sourcelist'}} = split(/\,/,$source);
	$obj->{'multisource'} = 1;
    }
# Source range
    elsif($source =~ /\-/)
    {
	my ($start,$stop) = split(/\-/,$source);

	my $result = check_ip($start);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $start\n\n";
	    $code = 1;
	}

	$result = check_ip($stop);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $stop\n\n";
	    $code = 1;
	}

	@{$obj->{sourcelist}} = get_ips($start,$stop);
	$obj->{'multisource'} = 1;
    }
    else
    {
	$obj->{'sourcelist'}->[0] = $source;
	my $result = check_ip($source);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $source\n\n";
	    $code = 1;
	}
    }
}


# Connection Config
# Es soll nur eine bestimmte Connection resettet werden
if($args{'c'})
{
    @{$obj->{'connection'}} = split(/\-/,$args{'c'});

    if(scalar(@{$obj->{connection}}) > 2)
    {
	croak("A connection looks like this: xxx.xxx.xxx.xxx-xxx.xxx.xxx.xxx\n");
    }

    for(@{$obj->{'connection'}})
    {
	my $result = check_ip($_);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $_\n\n";
	    $code = 1;
	}
    }
}



# Port Config
# Wir sollen nur Packete fuer einen bestimmten Port resetten
if($args{'p'})
{
    $port = $args{'p'};

	# Port List?
	if($port =~ /\,/)
	{
		@{$obj->{'port'}} = split(/\,/,$port);
		$obj->{'multiport'} = 1;
	}

	# Port Range?
	elsif($port =~ /\-/)
	{
		my($start,$stop) = split(/\-/,$port);

		for(my $i=$start; $i <= $stop; $i++)
		{
			push @{$obj->{'port'}}, $i;
		}

		$obj->{'multiport'} = 1;
	}

       # Ein einsamer Port
       else
       {
	   $obj{'port'}->[0] = $args{'p'};
       }
}

# Trat zwischen durch ein Fehler auf?
if($code == 1)
{
    return $code;
}
else
{
    return $obj;
}
}


# Print configuration
sub print
{
    my $obj = shift;

# Device 
    print "---:[ Device $obj->{'device'} ]:---\n\n";


# Port Config
    if(defined $obj->{'port'})
    {
	if($obj->multiport())
	{
	    print "---:[ Ports ";

	    for(@{$obj->{'port'}})
	    {
		print "$_ ";
	    }

	    print "]:---\n\n";
	}
	else
	{
	    print "---:[ Port " . $obj->{'port'}->[0] . " ]:---\n\n";
	}
    }


# Target Config
    if(defined $obj->{'targetlist'}->[0])
    {
	if($obj->multitarget())
	{
	    print "---:[ Targets ";

	    for(@{$obj->{'targetlist'}})
	    {
		print "$_ ";
	    }

	    print "]:---\n\n";
	}
	else
	{
	    print "---:[ Target $obj->{'targetlist'}->[0] ]:---\n\n";
	}
    }


# Source Config
    if(defined $obj->{'sourcelist'}->[0])
    {
	if($obj->multisource())
	{
	    print "---:[ Source ";

	    for(@{$obj->{'sourcelist'}})
	    {
		print "$_ ";
	    }

	    print "]:---\n\n";
	}
	else
	{
	    print "---:[ Source $obj->{'sourcelist'}->[0] ]:---\n\n";
	}
    }


# Connection
    if(defined $obj->{'connection'}->[0])
    {
	print "---:[ Connection $obj->{'connection'}->[0] <--> $obj->{'connection'}->[1] ]:---\n\n";
    }
}


# Change target
sub set_target
{
    my $obj = shift;
    my @host = @_;
    @{$obj->{'targetlist'}} = @host;
    return $obj;
}


# Change source
sub set_source
{
    my $obj = shift;
    my @host = @_;
    @{$obj->{'sourcelist'}} = @host;
    return $obj;
}

# Change connection
sub set_connection
{
    my $obj = shift;
    my $connection = shift;
    $obj->{'connection'} = $connection;
    return $obj;
}

# Change port
sub set_port
{
    my $obj = shift;
    my @port = @_;
    @{$obj->{'port'}} = @port;
    return $obj;
}


# Change device
sub set_device
{
    my $obj = shift;
    my $device = shift;
    $obj->{'device'} = $device;
    return $device;
}

# Set flags
sub set_flags
{
    my $obj = shift;
    my %flags = @_;
    $obj->{'syn'} = $flags{'syn'};
    $obj->{'ack'} = $flags{'ack'};
    $obj->{'rst'} = $flags{'rst'};
    $obj->{'fin'} = $flags{'fin'};
    $obj->{'psh'} = $flags{'psh'};
    $obj->{'urg'} = $flags{'urg'};
    return $obj;
}

# Set TCP options
sub set_tcpopt
{
    my $obj = shift;
    my %opts = @_;
    $obj->set_flags(%opts);
    $obj->{'win'} = $opts{'win'};
    $obj->{data} = $opts{data};
    $obj->{seq} = $opts{seq};
    $obj->{'ack_seq'} = $opts{'ack_seq'};
    $obj->{'src_port'} = $opts{'src_port'};
    $obj->{'dst_port'} = $opts{'dst_port'};
    $obj->{protocol} = "tcp";
    return $obj;
}

# Set UDP options
sub set_udpopt
{
    my $obj = shift;
    my %opts = @_;
    $obj->{'src_port'} = $opts{'src_port'};
    $obj->{'dst_port'} = $opts{'dst_port'};
    $obj->{data} = $opts{data};
    $obj->{protocol} = "udp";
    return $obj;
}

# Set ICMP options
sub set_icmpopt
{
    my $obj = shift;
    my %opts = @_;
    $obj->{type} = $opts{type};
    $obj->{code} = $opts{code};
    $obj->{gateway} = $opts{gateway};
    $obj->{mtu} = $opts{mtu};
    $obj->{data} = $opts{data};
    $obj->{protocol} = "icmp";
    return $obj;    
}

# Set IP options
sub set_ipopt
{
    my $obj = shift;
    my %opts = @_;
    $obj->{ttl} = $opts{ttl};
    $obj->{tos} = $opts{tos};
    $obj->{frag} = $opts{frag};
    $obj->{sourcelist}->[0] = $opts{spoof};
    $obj->{targetlist}->[0] = $opts{target};
    return $obj;    
}

# Set general packet options
sub set_opt
{
        my $obj = shift;
	my %opts = @_;

	$obj->set_ipopt(%opts);
	$obj->{protocol} = $opts{protocol};

	if($opts{protocol} eq "tcp")
	{
	    $obj->set_tcpopt(%opts);
	}
	elsif($opts{protocol} eq "udp")
	{
	    $obj->set_udpopt(%opts);
	}
	elsif($opts{protocol} eq "icmp")
	{
	    $obj->set_icmpopt(%opts);
	}

	return $obj;
}

# Extract target host(s)
sub get_target
{
    my $obj = shift;

    if($obj->{'multitarget'})
    {
	return $obj->{'targetlist'};
    }
    else
    {
	return $obj->{'targetlist'}->[0];
    }
}

# Extract source host(s)
sub get_source
{
    my $obj = shift;

    if($obj->{'multisource'})
    {
	return $obj->{'sourcelist'};
    }
    else
    {
	return $obj->{'sourcelist'}->[0];
    }
}


# Extract connection(s)
sub get_connection
{
    my $obj = shift;
    return $obj->{'connection'};
}


# Extract port(s)
sub get_port
{
    my $obj = shift;

    if($obj->{'multiport'})
    {
	return $obj->{'port'};
    }
    else
    {
	return $obj->{'port'}->[0];
    }
}

# Extract device
sub get_device
{
    my $obj = shift;
    return $obj->{'device'};
}

# Get IP options
sub get_ipopt
{
    my $obj = shift;

# Momentan noch kein Multisource, -target oder -port Support
    $opts{'spoof'} = $obj->{sourcelist}->[0];
    $opts{'target'} = $obj->{targetlist}->[0];
    $opts{'frag'} = $obj->{frag};
    $opts{'ttl'} = $obj->{ttl};
    $opts{tos} = $obj->{tos};

    return %opts;
}

# Get TCP options
sub get_tcpopt
{
    my $obj = shift;

    $opts{'src_port'} = $obj->{'src_port'};
    $opts{'dst_port'} = $obj->{'dst_port'};
    $opts{'syn'} = $obj->{syn};
    $opts{'ack'} = $obj->{ack};
    $opts{'rst'} = $obj->{rst};
    $opts{'fin'} = $obj->{fin};
    $opts{'psh'} = $obj->{psh};
    $opts{'urg'} = $obj->{urg};
    $opts{'seq'} = $obj->{seq};
    $opts{'ack_seq'} = $obj->{'ack_seq'};
    $opts{win} = $obj->{win};
    $opts{data} = $obj->{data};

    return %opts;
}

# Get UDP options
sub get_udpopt
{
    my $obj = shift;

    $opts{'src_port'} = $obj->{'src_port'};
    $opts{'dst_port'} = $obj->{'dst_port'}; 
    $opts{data} = $obj->{data};

    return %opts;
}

# Get ICMP options
sub get_icmpopt
{
    my $obj = shift;

    $opts{type} = $obj->{type};
    $opts{code} = $obj->{code};
    $opts{gateway} = $obj->{gateway};
    $opts{data} = $obj->{data};
    $opts{mtu} = $obj->{mtu};

    return %opts;
}

# Get general packet options
sub get_opt
{
    my $obj = shift;

    %opts = $obj->get_ipopt();

    if($obj->{protocol} eq "tcp")
    {
	%opts = $obj->get_tcpopt();
    }
    elsif($obj->{protocol} eq "udp")
    {
	%opts = $obj->get_udpopt();
    }
    elsif($obj->{protocol} eq "icmp")
    {
	%opts = $obj->get_icmpopt();
    }

    return %opts;
}

# extract the used protocol
sub get_protocol
{
    my $obj = shift;
    my $proto = shift;

    if($proto ne "")
    {
	if($obj->{protocol} eq $proto)
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }
    else
    {
	return $obj->{protocol};
    }
}

# extract the flags
sub get_flags
{
    my $obj = shift;
    my %opts;

    $opts{'syn'} = $obj->{syn};
    $opts{'ack'} = $obj->{ack};
    $opts{'rst'} = $obj->{rst};
    $opts{'fin'} = $obj->{fin};
    $opts{'psh'} = $obj->{psh};
    $opts{'urg'} = $obj->{urg};

    return %opts;
}

# Multitarget?
sub multitarget
{
    my $obj = shift;
    return $obj->{'multitarget'};
}

# Multisource?
sub multisource
{
    my $obj = shift;
    return $obj->{'multisource'};
}

# Multi port?
sub multiport
{
    my $obj = shift;
    return $obj->{'multiport'};
}


# Gibt es einen Connection Eintrag?
sub connection
{
    my $obj = shift;

    if($obj->{'connection'}->[0] ne "")
    {
	return 1;
    }
    else
    {
	return 0;
    }
}


# Gibt es einen Target Eintrag?
sub target
{
    my $obj = shift;

    if($obj->{'targetlist'}->[0] ne "")
    {
	return 1;
    }
    else
    {
	return 0;
    }
}

# Gibt es einen Source Eintrag?
sub source
{
    my $obj = shift;

    if($obj->{'sourcelist'}->[0] ne "")
    {
	return 1;
    }
    else
    {
	return 0;
    }
}


# Gibt es einen Port Eintrag?
sub port
{
    my $obj = shift;

    if($obj->{'port'}->[0] ne "")
    {
	return 1;
    }
    else
    {
	return 0;
    }
}


# Check address
sub check_address
{
    my $address = shift;
    my $code = 0;

# Source list
    if($address =~ /\,/g)
    {
	my @addresses = split(/\,/,$address);

	for(@addresses)
	{
	    my $result = check_ip($_);

	    if($result == 1)
	    {
		print ">>> BAD IP ADDRESS $_\n\n";
		$code = 1;
	    }
	}
    }
# Source range
    elsif($address =~ /\-/)
    {
	my ($start,$stop) = split(/\-/,$address);

	my $result = check_ip($start);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $start\n\n";
	    $code = 1;
	}

	$result = check_ip($stop);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $stop\n\n";
	    $code = 1;
	}
    }
    else
    {
	my $result = check_ip($address);

	if($result == 1)
	{
	    print ">>> BAD IP ADDRESS $address\n\n";
	    $code = 1;
	}
    }

    return $code;
}


# Check if ip address is correct
sub check_ip
{
    my $ip = shift;
    my @ip = split(/\./,$ip);
    my $code = 0;

# Mehr als 4 Byte?
if(scalar(@ip != 4))
{
    $code = 1;
}

# Ist ein Wert groesser als 256?
# Oder enthaelt er gar was anderes als Zahlen?
for(@ip)
{
    unless($_ =~ /(\d)+/)
    {
	$code = 1;
    }
    elsif($_ > 256)
    {
	$code = 1;
    }
}

return $code;
}


# Get ips
sub get_ips
{
    my $start = shift;
    my $stop = shift;
# Rechne die IP in eine Dezimalzahl um
	my @start = split(/\./,$start); 
	my @stop = split(/\./,$stop); 

	my $start_dec = (($start[0]*16777216)+($start[1]*65536)+($start[2]*256)+$start[3]);
	my $stop_dec = (($stop[0]*16777216)+($stop[1]*65536)+($stop[2]*256)+$stop[3]);


# Berechne Start / End IP
	while($start_dec < $stop_dec + 1)
	{
		my @bytes;
		my $rem;

# Rechne die Dezimal Zahl wieder in eine IP Adresse um
	    	$bytes[0] = int $start_dec/16777216;
	    	$rem = $start_dec % 16777216;
	    	$bytes[1] = int $rem/65536;
	    	$rem = $rem % 65536;
	    	$bytes[2] = int $rem/256;
	    	$rem = $rem % 256;
	    	$bytes[3] = $rem;
	  	my $ipaddr = join '.', @bytes;
		push @{$obj->{'target'}}, $ipaddr;
		$start_dec++;
	}
	
}


# Create a pcap expression from configuration
sub pcap
{
    my $obj = shift;
    my $pcap = "";
    my @conn = ('');

    if($obj->multitarget)
    {
	for(@{$obj->get_target})
	{
	    $pcap .= "host $_ or ";
	}

	chop $pcap;
	chop $pcap;
	chop $pcap;
    }
    elsif($obj->get_target ne "")
    {
	$pcap .= "host " . $obj->get_target . " ";
    }
    elsif(scalar(@{$obj->get_connection}) > 0)
    {
	@conn = @{$obj->get_connection};
	$pcap .= "host $conn[0] or host $conn[1] ";
    }

    if($pcap ne "")
    {
	$pcap .= "and ";
    }

    if($obj->multiport)
    {
	for(@{$obj->get_port})
	{
	    $pcap .= "port $_ or ";
	}

	chop $pcap;
	chop $pcap;
	chop $pcap;
    }
    elsif($obj->get_port ne "")
    {
	$pcap .= "port " . $obj->get_port . " ";
    }

    return $pcap;
}


###[ Parse a config file ]###

sub readcfg
{
    my $opt = shift;
    my $file = shift;
    my($command,$wert);
    my(@config);
    my(%parameter);

# Die Konfigurationsdatei wird gelesen
open (CONFIG,"<$file") || croak("Kann Configfile $file nicht einlesen!!!\n");


	while(<CONFIG>)
	{
		# Auskommentierte Zeilen auslassen
		if($_ =~ /^#/)
		{
		    next;
       		}
		# Leere Zeilen auch
       		elsif(length($_)<2)
       		{
		    next;
       		}
       		else
       		{
		    chomp $_;
		    ($command,$wert) = split(/\s+==\s+/,$_);

		    # Write a Array?
		    if($opt eq "a")
		    {
			push @config, $command;
			push @config, $wert;
		    }
		   else
		    {
		    	$parameter{$command}=$wert;
		    }
		}
	}
	
close(CONFIG);

if($opt eq "a")
{
    return @config;
}
else
{
    return %parameter;
}
}


# Generate a config object
# out of a config file
sub register_config
{
    my $obj = shift;
    my %args = @_;
    $obj->set_device($args{device});
    $obj->set_source($args{spoof});
    $obj->set_target($args{target});
    $obj->set_port($args{port});
    return $obj;
}

1;


###[ Thats the end folks ]###


__END__


###[ POD Documentation ]###

=pod
=head2 NAME

    Config.pm  --  P.A.T.H Configuration

=head2 SYNOPSIS

    use config;
    use Getopt::Std;

    getopts('h:c:p:i:',%args);
    $cfg = config->new();
    if($cfg->check(%args) == 1)
    {
	exit(1);
    }
    
    if($cfg->connection)
    {
	@connect = $cfg->connection;
	print "Connection $connection[0] <--> $connection[1]\n";
    }

   if($cfg->multiport)
   {
	@ports = $cfg->get_port;

	print "Ports: ";

	for(@ports)
	{
	   print;
	}

	print "\n";
   }


=head2 DESCRIPTION

   This module checks the configuration for P.A.T.H. scripts.
   It parses the parameter like host (lists,ranges), connections,
   port (lists,ranges) and device.
   You can set and get config parameters.
   This module can also read and parse a configuration file.


=head2 METHODS

   new
   check
   print
   get_device
   get_target
   get_source
   get_connection
   get_port
   get_flags
   get_ipopt
   get_tcpopt
   get_udpopt
   get_icmpopt
   get_opt
   set_flags
   set_ipopt
   set_tcpopt
   set_udpopt
   set_cmpopt
   set_opt
   set_device
   set_target
   set_source
   set_connection
   set_port
   multiport
   multitarget
   multisource
   target
   source
   port
   connection
   pcap
   readcfg
   register_config


=head2 Description of methods

=item B<new()>

   $cfg = config->new()

   This creates a new config object.

=item B<check()>

   $cfg->check(%args);

   This method parses and checks the parameter in the hash %args.
   It return a value of 1 if there was an error.
   %args can contain the following things:
   i: device
   c: connection (target1-target2)
   h: host (list: host1,host2 // range host1-host3)
   p: port (list or range)


=item B<print()>

   $cfg->print();

   This will print the configuration to STDOUT.


=item B<get_device()>

   $cfg->get_device

   This method returns the configured device.
   All the other get_* methods do the same.


=item B<set_device()>

   $cfg->set_device("eth1");

   This method changes the device configuration and
   returns the new object.
   All teh other set_* methods are used similar and
   do the same.


=item B<set_opt>

    $cfg->set_opt(%config);

    This method sets the packet options like TCP flags, 
    window size and so on. There is also a set_* method
    for each supported protocol.
    See the section configuration hash for a description
    of the hash %config.

=item B<get_opt>
    
    %config = $cfg->get_opt();

    This method can be used to receive all the packet
    option from the config object. There is also a get_*
    method for each supported protcol.
    See the section configuration hash for a description
    of the hash %config.

=item B<multitarget()>

   $cfg->multitarget;

   This method will return the boolean value true if the
   configuration contains more than one host.
   The multisource and multiport method is used the same way.


=item B<connection()>

   $cfg->connection;

   This method return the boolean value true if a connection
   is found in the configuration.
   The port and host method are used the same way.

=item B<pcap()>

   $string = $cfg->pcap();

   Create a pcap expression from the configuration like:
   host 192.168.1.1 and port 23 or port 21

=item B<readcfg()>

   %config = config::readcfg($opt,$file);

   This function will parse a config file like this:
   parameter == value

  opt: a - Function return an array
       h - Function returns a hash (default)

  file: Contains the path to the config file.

=item B<register_config()>

    $cfg->register_config(%config);

    This method stores the default values like target
    and device into the config object.

=item B<The configuration hash>

    The configuration hash has the following values
    (depeding on which protocol should be configured):
    $opts{'protocol'}
    $opts{'spoof'}
    $opts{'target'}
    $opts{'frag'}
    $opts{'ttl'}
    $opts{tos}
    $opts{'src_port'}
    $opts{'dst_port'}
    $opts{'syn'}
    $opts{'ack'}
    $opts{'rst'}
    $opts{'fin'}
    $opts{'psh'}
    $opts{'urg'}
    $opts{'seq'}
    $opts{'ack_seq'}
    $opts{'win'}
    $opts{'data'}
    $opts{'type'}
    $opts{'code'}
    $opts{'gateway'}
    $opts{'mtu'}
    
=head2 BUGS

  Currently there are no known bugs...
  Please send bug reports to bugs@crazydj.de


=head2 AUTHOR

   Bastian Ballmann [ bytebeater@crazydj.de ]
   http://www.crazydj.de


=head2 COPYRIGHT

   This module is free software.
   It is licensed under GPL.

